//
//  SNTRYSideMenuController.h
//
//  Created by Warren Merrifield on 17/08/2012.
//  Copyright (c) 2012 Lone Sentry Ltd. All rights reserved.
//
//	Permission is hereby granted, free of charge, to any person obtaining a
//	copy of this software and associated documentation files (the "Software"),
//	to deal in the Software without restriction, including without limitation
//	the rights to use, copy, modify, merge, publish, distribute, sublicense,
//	and/or sell copies of the Software, and to permit persons to whom the
//	Software is furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
//	IN THE SOFTWARE.
//

#import <UIKit/UIKit.h>

#ifndef __IPHONE_5_0
#warning "This project uses features only available in iOS SDK 5.0 and later."
#endif

/**** Constants ****/
typedef enum
{
	SNTRYSideMenuControllerMenuSideLeft,
	SNTRYSideMenuControllerMenuSideRight,
} SNTRYSideMenuControllerMenuSide;

/**** Notifications ****/
extern NSString *const SNTRYSideMenuControllerDelegateWillShowViewController;
extern NSString *const SNTRYSideMenuControllerDelegateDidShowViewController;
extern NSString *const SNTRYSideMenuControllerDelegateDidHideViewController;
extern NSString *const SNTRYSideMenuControllerViewControllerKey;

/**** Delegate Protocol ****/
@class SNTRYSideMenuController;
@protocol SNTRYSideMenuControllerDelegate <NSObject>
@optional
-(void)slidingMenusViewController:(SNTRYSideMenuController*)viewController
		   willShowViewController:(UIViewController*)viewControllerToShow;
-(void)slidingMenusViewController:(SNTRYSideMenuController*)viewController
			didShowViewController:(UIViewController*)viewControllerToShow;
-(BOOL)slidingMenusViewController:(SNTRYSideMenuController*)viewController
		 shouldHideViewController:(UIViewController*)viewControllerToHide;
-(void)slidingMenusViewController:(SNTRYSideMenuController*)viewController
			didHideViewController:(UIViewController*)viewControllerToHide;

@end

/**** Class Interface ****/
@interface SNTRYSideMenuController : UIViewController

@property (nonatomic, weak) IBOutlet id<SNTRYSideMenuControllerDelegate>	delegate;
@property (nonatomic, strong) IBOutlet UIViewController*					mainViewController;
@property (nonatomic) CGFloat												menuWidth;

-(id)initWithMainViewController:(UIViewController*)viewController;

-(void)setMenuWidth:(CGFloat)newWidth animated:(BOOL)animated;

// Shows and hides the given view controller on the given side.
// Automatically hides any other controller that is visible.
-(void)showMenuViewController:(UIViewController*)controller
					   onSide:(SNTRYSideMenuControllerMenuSide)side
					 animated:(BOOL)animated;
-(void)hideMenuViewControllerAnimated:(BOOL)animated;


@end
