//
//  SNTRYSideMenuController.m
//
//  Created by Warren Merrifield on 07/06/2012.
//  Copyright (c) 2012 Lone Sentry Ltd. All rights reserved.
//
//	Permission is hereby granted, free of charge, to any person obtaining a
//	copy of this software and associated documentation files (the "Software"),
//	to deal in the Software without restriction, including without limitation
//	the rights to use, copy, modify, merge, publish, distribute, sublicense,
//	and/or sell copies of the Software, and to permit persons to whom the
//	Software is furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
//	IN THE SOFTWARE.
//

#import <QuartzCore/QuartzCore.h>
#import "SNTRYSideMenuController.h"

#define LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN	(1)

NSString *const SNTRYSideMenuControllerDelegateWillShowViewController	= @"uk.co.lonesentry.SNTRYSideMenuControllerDelegateWillShowViewController";
NSString *const SNTRYSideMenuControllerDelegateDidShowViewController	= @"uk.co.lonesentry.SNTRYSideMenuControllerDelegateDidShowViewController";
NSString *const SNTRYSideMenuControllerDelegateDidHideViewController	= @"uk.co.lonesentry.SNTRYSideMenuControllerDelegateDidHideViewController";
NSString *const SNTRYSideMenuControllerViewControllerKey				= @"viewController";

static const CGFloat SNTRYSideMenuControllerDefaultMenuWidth = 260.0f;

/**** Private Interface ****/
@interface SNTRYSideMenuController ()

@property (nonatomic, strong) UIViewController*			menuViewController;
@property (nonatomic, strong) UIView*					mainView;
@property (nonatomic, strong) UIView*					sideView;
@property (nonatomic, strong) UIImageView*				screenshotView;
@property (nonatomic) SNTRYSideMenuControllerMenuSide	menuSide;

@end

/**** Implementation ****/
@implementation SNTRYSideMenuController

#pragma mark - Object Lifecycle

-(id)init
{
	NSAssert(NO, @"You must use initWithMainViewController: to initialise objects of this class");
	[self doesNotRecognizeSelector:_cmd];
	return nil;

}

-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
	NSAssert(NO, @"You must use initWithMainViewController: to initialise objects of this class");
	[self doesNotRecognizeSelector:_cmd];	
	return nil;
}

-(id)initWithMainViewController:(UIViewController*)viewController
{
	if(viewController == nil)
	{
		return nil;
	}
	
	self = [super initWithNibName:nil bundle:nil];
	if(self)
	{
		_menuWidth = SNTRYSideMenuControllerDefaultMenuWidth;
		_mainViewController = viewController;
		[self addChildViewController:viewController];
	}
	return self;
}

-(id)initWithCoder:(NSCoder *)aDecoder
{
	self = [super initWithCoder:aDecoder];
	if(self)
	{
		_menuWidth = SNTRYSideMenuControllerDefaultMenuWidth;
	}
	return self;
}

#pragma mark - Orientation

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN

-(void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
	[super willRotateToInterfaceOrientation:toInterfaceOrientation
								   duration:duration];
	[self.mainViewController willRotateToInterfaceOrientation:toInterfaceOrientation
													 duration:duration];
	[self.menuViewController willRotateToInterfaceOrientation:toInterfaceOrientation
													 duration:duration];
}

-(void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
	[super willAnimateRotationToInterfaceOrientation:toInterfaceOrientation
											duration:duration];
	[self.mainViewController willAnimateRotationToInterfaceOrientation:toInterfaceOrientation
															  duration:duration];
	[self.menuViewController willAnimateRotationToInterfaceOrientation:toInterfaceOrientation
															  duration:duration];
}

-(void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
	[super didRotateFromInterfaceOrientation:fromInterfaceOrientation];
	[self.mainViewController didRotateFromInterfaceOrientation:fromInterfaceOrientation];
	[self.menuViewController didRotateFromInterfaceOrientation:fromInterfaceOrientation];
}

#endif

#pragma mark - Mutators

-(void)setMainViewController:(UIViewController *)newViewController
{
	NSParameterAssert(newViewController);
	if(newViewController == _mainViewController ||
	   [newViewController isEqual:_mainViewController])
	{
		// Early out
		return;
	}
	
	// Now replace the old child view controller with this new one
	[_mainViewController removeFromParentViewController];
	if(newViewController)
	{
		[self addChildViewController:newViewController];
	}
	_mainViewController = newViewController;
}

-(void)setMenuWidth:(CGFloat)newWidth
{
	[self setMenuWidth:newWidth animated:NO];
}

-(void)setMenuWidth:(CGFloat)newWidth animated:(BOOL)animated
{
	// Limit the width to a sensible range
	newWidth = MAX(newWidth, 0.0f);
	newWidth = MIN(newWidth, 300.0f);
	
	// Early out if the width remains the same
	if(newWidth == _menuWidth)
	{
		return;
	}
	
	// Are we getting bigger or not?
	BOOL gettingWider = YES;
	if(newWidth < _menuWidth)
	{
		gettingWider = NO;
	}
	_menuWidth = newWidth;
	
	// If the menu is open we need to relayout
	if(self.menuViewController && [self isViewLoaded])
	{
		// Work out our new destination frames
		CGRect	destMainFrame,
				destMenuFrame;
		if(self.menuSide == SNTRYSideMenuControllerMenuSideLeft)
		{
			destMainFrame = [self SNTRY_mainFrameLeftOpen];
			destMenuFrame = [self SNTRY_menuFrameLeftOpen];
		}
		else
		{
			destMainFrame = [self SNTRY_mainFrameRightOpen];
			destMenuFrame = [self SNTRY_menuFrameRightOpen];
		}
		
		if(animated)
		{
			[UIView animateWithDuration:0.25
							 animations:^{
								 self.screenshotView.frame = destMainFrame;
								 if(gettingWider == YES)
								 {
									 self.sideView.frame = destMenuFrame;
								 }
							 }
							 completion:^(BOOL finished){
								 // If the menu is getting narrower, the set the
								 // menu frame at the end to prevent glitches.
								 if(gettingWider == NO)
								 {
									 self.sideView.frame = destMenuFrame;
								 }
							 }];
		}
		else
		{
			self.screenshotView.frame = destMainFrame;
			self.sideView.frame = destMenuFrame;
		}
	}
}


#pragma mark - View Lifecycle

-(void)viewDidLoad
{
	[super viewDidLoad];
	
	// Add the mainViewController's view to this view
	NSParameterAssert(self.mainViewController);
	self.mainView = self.mainViewController.view;
	[self.view addSubview:self.mainView];
}

#pragma mark - Accessors

-(CGRect)SNTRY_menuFrameLeftOpen
{
	return CGRectMake(0.0f, 0.0f, self.menuWidth, self.view.bounds.size.height);
}

-(CGRect)SNTRY_mainFrameLeftOpen
{
	return CGRectMake(self.menuWidth, 0.0f, self.view.bounds.size.width, self.view.bounds.size.height);
}

-(CGRect)SNTRY_menuFrameRightOpen
{
	NSAssert(self.view.bounds.size.width > self.menuWidth, @"Menu width too wide!");
	CGFloat mainWidth = self.view.bounds.size.width - self.menuWidth;
	return CGRectMake(mainWidth, 0.0f, self.menuWidth, self.view.bounds.size.height);
}

-(CGRect)SNTRY_mainFrameRightOpen
{
	return CGRectMake(-self.menuWidth, 0.0f, self.view.bounds.size.width, self.view.bounds.size.height);
}


#pragma mark - Actions

- (BOOL)automaticallyForwardAppearanceAndRotationMethodsToChildViewControllers
{
#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN
	return NO;
#else
	return YES;
#endif
}

-(CAGradientLayer*)SNTRY_shadowGradientLayer
{
	CAGradientLayer* gradientLayer = [CAGradientLayer layer];
	
	UIColor *colorOne	= [UIColor colorWithWhite:0.0f alpha:0.7f];
	UIColor *colorTwo	= [UIColor colorWithWhite:0.0f alpha:0.0f];
	NSArray *colors		= @[(id)colorOne.CGColor, (id)colorTwo.CGColor];
	gradientLayer.colors = colors;
	
	NSNumber *stopOne	= @(0.0f);
	NSNumber *stopTwo	= @(1.0f);
	NSArray  *locations	= @[stopOne, stopTwo];
	gradientLayer.locations = locations;
	
	return gradientLayer;
}

-(void)SNTRY_createScreenshotViewForSide:(SNTRYSideMenuControllerMenuSide)side
{
	NSAssert(self.screenshotView == nil, @"Should not already have a screenshot view at this point");
	
	CGRect rect = [self.mainView bounds];
	UIGraphicsBeginImageContextWithOptions(rect.size,YES,0.0f);
	CGContextRef context = UIGraphicsGetCurrentContext();
	[self.mainView.layer renderInContext:context];
	UIImage *capturedImage = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	
	// Take a snapshot of the main screen now
	self.screenshotView = [[UIImageView alloc] initWithImage:capturedImage];
	self.screenshotView.opaque = YES;
	self.screenshotView.userInteractionEnabled = YES;
	self.screenshotView.isAccessibilityElement = YES;
	self.screenshotView.accessibilityLabel = NSLocalizedString(@"Close", nil);
	self.screenshotView.accessibilityHint = NSLocalizedString(@"Closes the side menu.", nil);
	self.screenshotView.accessibilityTraits = UIAccessibilityTraitButton;
	UITapGestureRecognizer* recognizer = [[UITapGestureRecognizer alloc] initWithTarget:self
																				 action:@selector(SNTRY_closeMenu:)];
	[self.screenshotView addGestureRecognizer:recognizer];
	
	// Add a shadow gradient layer to the sides of this view
	CAGradientLayer* edgeLayer = [self SNTRY_shadowGradientLayer];
	if(side == SNTRYSideMenuControllerMenuSideLeft)
	{
		edgeLayer.startPoint = CGPointMake(1.0, 0.5);
		edgeLayer.endPoint = CGPointMake(0.0, 0.5);
		edgeLayer.frame = CGRectMake(-5.0f, 0.0f, 5.0f, rect.size.height);
	}
	else
	{
		edgeLayer.startPoint = CGPointMake(0.0, 0.5);
		edgeLayer.endPoint = CGPointMake(1.0, 0.5);
		edgeLayer.frame = CGRectMake(rect.size.width, 0.0f, 5.0f, rect.size.height);
	}
	[self.screenshotView.layer addSublayer:edgeLayer];
}

-(void)showMenuViewController:(UIViewController*)viewController
					   onSide:(SNTRYSideMenuControllerMenuSide)side
					 animated:(BOOL)animated
{
	NSParameterAssert(viewController);
	NSAssert(side == SNTRYSideMenuControllerMenuSideLeft ||
			 side == SNTRYSideMenuControllerMenuSideRight,
			 @"side parameter must be either left or right.");
	
	if([self.menuViewController isEqual:viewController] && side == self.menuSide)
	{
		// Early out; already visible
		return;
	}
	
	// If we already have a menu view controller visible, then try to close that first
	if(self.menuViewController)
	{
		[self hideMenuViewControllerAnimated:animated];
	}
	
	// Inform everyone of the change
	if([self.delegate respondsToSelector:@selector(slidingMenusViewController:willShowViewController:)])
	{
		[self.delegate slidingMenusViewController:self
						   willShowViewController:self.menuViewController];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:SNTRYSideMenuControllerDelegateWillShowViewController
														object:self
													  userInfo:@{SNTRYSideMenuControllerViewControllerKey: viewController}];
	
	// Take a snapshot of the main screen now
	[self SNTRY_createScreenshotViewForSide:side];
	
	// OK, now do the showing
	self.menuViewController = viewController;
#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN
	[self.menuViewController viewWillAppear:animated];
#endif
	[self addChildViewController:self.menuViewController];
	self.sideView = self.menuViewController.view;
	[self.view addSubview:self.sideView];
	
	// And replace the main view with our screenshot
	[self.mainView removeFromSuperview];
	[self.view addSubview:self.screenshotView];
	
	// Work out our new destination frames
	CGRect	startMenuFrame,
			destMainFrame,
			destMenuFrame;
	self.menuSide = side;
	if(self.menuSide == SNTRYSideMenuControllerMenuSideLeft)
	{
		destMainFrame = [self SNTRY_mainFrameLeftOpen];
		destMenuFrame = [self SNTRY_menuFrameLeftOpen];
		startMenuFrame = CGRectOffset(destMenuFrame, -destMenuFrame.size.width, 0.0f);
	}
	else {
		destMainFrame = [self SNTRY_mainFrameRightOpen];
		destMenuFrame = [self SNTRY_menuFrameRightOpen];
		startMenuFrame = CGRectOffset(destMenuFrame, destMenuFrame.size.width, 0.0f);
	}
	
	// Update the frames
	if(animated)
	{
		self.sideView.frame = startMenuFrame;
		__weak SNTRYSideMenuController* blockSelf = self;
		[UIView animateWithDuration:0.25
						 animations:^{
							 blockSelf.screenshotView.frame = destMainFrame;
							 blockSelf.sideView.frame = destMenuFrame;
						 }
						 completion:^(BOOL finished){
#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN
							 [self.menuViewController viewDidAppear:YES];
#endif
							 [self.menuViewController didMoveToParentViewController:self];
							 
							 // Announce accessability changes
							 UIAccessibilityPostNotification(UIAccessibilityAnnouncementNotification,
															 [NSString localizedStringWithFormat:@"%@ opened.", self.menuViewController.title]);
						 }];
	}
	else
	{
		self.screenshotView.frame = destMainFrame;
		self.sideView.frame = destMenuFrame;
#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN
		[self.menuViewController viewDidAppear:NO];
#endif
		[self.menuViewController didMoveToParentViewController:self];

		// Announce accessability changes
		UIAccessibilityPostNotification(UIAccessibilityAnnouncementNotification,
										[NSString localizedStringWithFormat:@"%@ opened.", self.menuViewController.title]);		
	}
	
	
	// Inform everyone of the change
	if([self.delegate respondsToSelector:@selector(slidingMenusViewController:didShowViewController:)])
	{
		[self.delegate slidingMenusViewController:self
							didShowViewController:self.menuViewController];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:SNTRYSideMenuControllerDelegateDidShowViewController
														object:self
													  userInfo:@{SNTRYSideMenuControllerViewControllerKey: self.menuViewController}];
}

-(void)SNTRY_closeMenu:(UIGestureRecognizer*)recognizer
{
	[self hideMenuViewControllerAnimated:YES];
}

-(void)hideMenuViewControllerAnimated:(BOOL)animated
{
	// Early out if there isn't a menu view controller
	if(self.menuViewController == nil)
	{
		return;
	}
	NSAssert([self.screenshotView window]!=nil, @"The screenshot view must be in the view hierarchy at this point");
	
	// Check with the delegate to see if we should try to close this
	// view controller
	if([self.delegate respondsToSelector:@selector(slidingMenusViewController:shouldHideViewController:)])
	{
		BOOL shouldHide = [self.delegate slidingMenusViewController:self
										   shouldHideViewController:self.menuViewController];
		if(shouldHide == NO)
		{
			// Exit now
			return;
		}
	}
	
	// OK, do the hiding
#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN
	[self.menuViewController viewWillDisappear:animated];
#endif
	[self.menuViewController willMoveToParentViewController:nil];
	if(animated)
	{
		CGRect destMenuFrame = self.sideView.frame;
		if(self.menuSide == SNTRYSideMenuControllerMenuSideLeft)
		{
			destMenuFrame = CGRectOffset(destMenuFrame, -destMenuFrame.size.width, 0.0f);
		}
		else
		{
			destMenuFrame = CGRectOffset(destMenuFrame, +destMenuFrame.size.width, 0.0f);
		}
		
		__weak SNTRYSideMenuController* blockSelf = self;
		[UIView animateWithDuration:0.25
						 animations:^{
							 blockSelf.screenshotView.frame = self.view.bounds;
							 blockSelf.sideView.frame = destMenuFrame;
						 }
						 completion:^(BOOL finished){
							 [blockSelf SNTRY_informOfHide:YES];
						 }];
	}
	else
	{
		self.screenshotView.frame = self.view.bounds;
		self.sideView.frame = CGRectZero;
		[self SNTRY_informOfHide:NO];
	}
}

-(void)SNTRY_informOfHide:(BOOL)animated
{
	// Announce accessability changes
	UIAccessibilityPostNotification(UIAccessibilityAnnouncementNotification,
									[NSString localizedStringWithFormat:@"%@ closed.", self.menuViewController.title]);
	
	// Done @ end of animation
	[self.sideView removeFromSuperview];
#ifdef LSSLIDINGMENUS_MANUAL_FORWARDING_TO_CHILDREN
	[self.menuViewController viewDidDisappear:animated];
#endif
	[self.menuViewController removeFromParentViewController];
	
	// Replace the screenshot view with the main view again
	[self.screenshotView removeFromSuperview];
	[self.view addSubview:self.mainView];
	self.screenshotView = nil;
	
	// Inform everyone of the change
	if([self.delegate respondsToSelector:@selector(slidingMenusViewController:didHideViewController:)])
	{
		[self.delegate slidingMenusViewController:self
							didHideViewController:self.menuViewController];
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:SNTRYSideMenuControllerDelegateDidHideViewController
														object:self
													  userInfo:@{SNTRYSideMenuControllerViewControllerKey: self.menuViewController}];
	
	self.menuViewController = nil;
	self.sideView = nil;
}

@end
